/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.gui.options;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;


/**
 * @author Jeff Michaud
 * Displays all option panels as a set of tabs
 * Pressing Ok or Cancel buttons calls the respective ok or cancel
 * routines in each of the panels (tabs).
 */
public class ShrimpOptionsDialog extends JDialog {
	
	private JTabbedPane jTabbedPane;
	private Vector tabs;
	private JButton btnCancel;
	private JButton btnOK;

	public ShrimpOptionsDialog(Frame parent, boolean modal) {
		super(parent, "Shrimp Options", modal);
		this.jTabbedPane = new JTabbedPane();
		this.tabs = new Vector();
		this.btnOK = new JButton("  OK  ");
		this.btnCancel = new JButton("Cancel");

		initComponents();
	}
	
	public int getSelectedTabIndex() {
		return jTabbedPane.getSelectedIndex();
	}
	
	public void setSelectedTabIndex(int index) {
		jTabbedPane.setSelectedIndex(index);
	}
	
    private void initComponents() {
		this.getContentPane().setLayout(new BorderLayout());
		this.getContentPane().add(jTabbedPane, BorderLayout.CENTER);
		this.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		
		JPanel pnl = new JPanel(new FlowLayout(FlowLayout.CENTER));
		pnl.add(btnOK);
		pnl.add(btnCancel);
		this.getContentPane().add(pnl, BorderLayout.SOUTH);
		
		//Go through each of the tabs and run the ok method
		btnOK.addActionListener ( new ActionListener () {
			public void actionPerformed (ActionEvent e) {
				for (Iterator iter = tabs.iterator(); iter.hasNext();) {
					ShrimpOptions element = (ShrimpOptions) iter.next();
					element.ok();
				}
				dispose();
			}
		});

		//Go through each of the tabs and run the cancel method
		btnCancel.addActionListener ( new ActionListener () {
			public void actionPerformed (ActionEvent e) {
				for (Iterator iter = tabs.iterator(); iter.hasNext();) {
					ShrimpOptions element = (ShrimpOptions) iter.next();
					element.cancel();
				}
				dispose();
			}
		});

		pack();
		setSize(600, 525);
		getRootPane().setDefaultButton(btnOK);               		
		
		// put in center of screen
		Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
		int x = (int)(screen.getWidth() - this.getWidth()) / 2;
		int y = (int)(screen.getHeight() - this.getHeight()) / 2;
		setLocation (x,y);
    }
    
	/**
	 * Method addTab
	 * Adds an option tab to this options dialog
	 * @param tabName
	 * @param tab
	 */
    public void addTab(String tabName, JPanel tab) {    	
    	this.jTabbedPane.addTab(tabName, tab);
    	tabs.add(tab);
    }
        
}
